@@ -1,14 +1,18 @@
 """Modified JSON API v1.0 for OPTIMADE API"""
 # pylint: disable=no-self-argument,no-name-in-module
-from pydantic import Field, root_validator, BaseModel, AnyHttpUrl, EmailStr
+from enum import Enum
+
+from pydantic import Field, root_validator, BaseModel, AnyHttpUrl, AnyUrl, EmailStr
 from typing import Optional, Union, List
 
 from datetime import datetime
 
 from . import jsonapi
+from .utils import SemanticVersion
 
 
 __all__ = (
+    "DataType",
     "ResponseMetaQuery",
     "Provider",
     "ImplementationMaintainer",
@@ -23,6 +27,102 @@ __all__ = (
 )
 
 
+class DataType(Enum):
+    """Optimade Data Types
+
+See the section "Data types" in the OPTIMADE API specification for more information.
+    """
+
+    STRING = "string"
+    INTEGER = "integer"
+    FLOAT = "float"
+    BOOLEAN = "boolean"
+    TIMESTAMP = "timestamp"
+    LIST = "list"
+    DICTIONARY = "dictionary"
+    UNKNOWN = "unknown"
+
+    @classmethod
+    def get_values(cls):
+        """Get OPTIMADE data types (enum values) as a (sorted) list"""
+        return sorted((_.value for _ in cls))
+
+    @classmethod
+    def from_python_type(cls, python_type: Union[type, str, object]):
+        """Get OPTIMADE data type from a Python type"""
+        mapping = {
+            "bool": cls.BOOLEAN,
+            "int": cls.INTEGER,
+            "float": cls.FLOAT,
+            "complex": None,
+            "generator": cls.LIST,
+            "list": cls.LIST,
+            "tuple": cls.LIST,
+            "range": cls.LIST,
+            "hash": cls.INTEGER,
+            "str": cls.STRING,
+            "bytes": cls.STRING,
+            "bytearray": None,
+            "memoryview": None,
+            "set": cls.LIST,
+            "frozenset": cls.LIST,
+            "dict": cls.DICTIONARY,
+            "dict_keys": cls.LIST,
+            "dict_values": cls.LIST,
+            "dict_items": cls.LIST,
+            "NoneType": cls.UNKNOWN,
+            "None": cls.UNKNOWN,
+            "datetime": cls.TIMESTAMP,
+            "date": cls.TIMESTAMP,
+            "time": cls.TIMESTAMP,
+            "datetime.datetime": cls.TIMESTAMP,
+            "datetime.date": cls.TIMESTAMP,
+            "datetime.time": cls.TIMESTAMP,
+        }
+
+        if isinstance(python_type, type):
+            python_type = python_type.__name__
+        elif isinstance(python_type, object):
+            if str(python_type) in mapping:
+                python_type = str(python_type)
+            else:
+                python_type = type(python_type).__name__
+
+        return mapping.get(python_type, None)
+
+    @classmethod
+    def from_json_type(cls, json_type: str):
+        """Get OPTIMADE data type from a named JSON type"""
+        mapping = {
+            "string": cls.STRING,
+            "integer": cls.INTEGER,
+            "number": cls.FLOAT,  # actually includes both integer and float
+            "object": cls.DICTIONARY,
+            "array": cls.LIST,
+            "boolean": cls.BOOLEAN,
+            "null": cls.UNKNOWN,
+            # OpenAPI "format"s:
+            "double": cls.FLOAT,
+            "float": cls.FLOAT,
+            "int32": cls.INTEGER,
+            "int64": cls.INTEGER,
+            "date": cls.TIMESTAMP,
+            "date-time": cls.TIMESTAMP,
+            "password": cls.STRING,
+            "byte": cls.STRING,
+            "binary": cls.STRING,
+            # Non-OpenAPI "format"s, but may still be used by pydantic/FastAPI
+            "email": cls.STRING,
+            "uuid": cls.STRING,
+            "uri": cls.STRING,
+            "hostname": cls.STRING,
+            "ipv4": cls.STRING,
+            "ipv6": cls.STRING,
+        }
+
+        return mapping.get(json_type, None)
+
+
 class OptimadeError(jsonapi.Error):
     """detail MUST be present"""
 
@@ -34,16 +134,14 @@ class OptimadeError(jsonapi.Error):
 
 class Warnings(OptimadeError):
     """OPTIMADE-specific warning class based on OPTIMADE-specific JSON API Error.
-    From the specification:
 
-        A warning resource object is defined similarly to a JSON API
-        error object, but MUST also include the field type, which MUST
-        have the value "warning". The field detail MUST be present and
-        SHOULD contain a non-critical message, e.g., reporting
-        unrecognized search attributes or deprecated features.
+From the specification:
 
-    Note: Must be named "Warnings", since "Warning" is a built-in Python class.
-    """
+A warning resource object is defined similarly to a JSON API error object, but MUST also include the field type, which MUST have the value "warning".
+The field detail MUST be present and SHOULD contain a non-critical message, e.g., reporting unrecognized search attributes or deprecated features.
+
+Note: Must be named "Warnings", since "Warning" is a built-in Python class.
+"""
 
     type: str = Field(
         "warning", const=True, description='Warnings must be of type "warning"'
@@ -109,7 +207,7 @@ class Implementation(BaseModel):
         None, description="version string of the current implementation"
     )
 
-    source_url: Optional[AnyHttpUrl] = Field(
+    source_url: Optional[AnyUrl] = Field(
         None,
         description="URL of the implementation source, either downloadable archive or version control system",
     )
@@ -132,29 +230,27 @@ class ResponseMeta(jsonapi.Meta):
     """
 
     query: ResponseMetaQuery = Field(
-        ..., description="information on the query that was requested"
+        ..., description="Information on the Query that was requested"
     )
 
-    api_version: str = Field(
-        ...,
-        description="a string containing the version of the API "
-        "implementation, e.g. v0.9.5",
+    api_version: SemanticVersion = Field(
+        ..., description="A string containing the version of the API implementation.",
     )
 
     time_stamp: datetime = Field(
         ...,
-        description="a string containing the date and time at which the query was exexcuted",
+        description="A timestamp containing the date and time at which the query was executed.",
     )
 
     data_returned: int = Field(
         ...,
-        description="an integer containing the number of data objects "
-        "returned for the query.",
+        description="An integer containing the total number of data resource objects returned for the current `filter` query, independent of pagination.",
         ge=0,
     )
 
     more_data_available: bool = Field(
-        ..., description="`false` if all data has been returned, and `true` " "if not."
+        ...,
+        description="`false` if all data resource objects for this `filter` query have been returned in the response or if it is the last page of a paginated response, and `true` otherwise.",
     )
 
     provider: Provider = Field(
@@ -163,8 +259,7 @@ class ResponseMeta(jsonapi.Meta):
 
     data_available: Optional[int] = Field(
         None,
-        description="an integer containing the total number of data "
-        "objects available in the database",
+        description="An integer containing the total number of data resource objects available in the database for the endpoint.",
     )
 
     last_id: Optional[str] = Field(
@@ -181,11 +276,11 @@ class ResponseMeta(jsonapi.Meta):
 
     warnings: Optional[List[Warnings]] = Field(
         None,
-        description="List of warning resource objects representing non-critical errors or warnings. "
-        "A warning resource object is defined similarly to a JSON API error object, but MUST also include the field type, "
-        'which MUST have the value "warning". The field detail MUST be present and SHOULD contain a non-critical message, '
-        "e.g., reporting unrecognized search attributes or deprecated features. The field status, representing a HTTP "
-        "response status code, MUST NOT be present for a warning resource object. This is an exclusive field for error resource objects.",
+        description="""A list of warning resource objects representing non-critical errors or warnings.
+A warning resource object is defined similarly to a [JSON API error object](http://jsonapi.org/format/1.0/#error-objects), but MUST also include the field `type`, which MUST have the value `"warning"`.
+The field `detail` MUST be present and SHOULD contain a non-critical message, e.g., reporting unrecognized search attributes or deprecated features.
+The field `status`, representing a HTTP response status code, MUST NOT be present for a warning resource object.
+This is an exclusive field for error resource objects.""",
         uniqueItems=True,
     )
 
@@ -208,7 +303,7 @@ class Success(jsonapi.Response):
 
         # errors MUST be skipped
         if values.get("errors", None) is not None:
-            raise ValueError(f"'errors' MUST be skipped for a successful response")
+            raise ValueError("'errors' MUST be skipped for a successful response")
 
         return values
 
